unit MainUnit;

{$mode objfpc}{$H+}

{ TODO:
- parallax, particles
- Ship combat
- Set ship armor and stats by sType
- Grid
- Ship construction
- Main menu
- Game saving and loading
}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  dglOpenGL, nxTypes, nxGL, nxBass, GameUnit, types, nxGLExtra;

const GS = 40; // Grid size (GS x GS)

type
  TUIButtonType = (btAttack, btComm, btWarp, btShipyard, btMap);
  TFocusType = (ftNone, ftButton, ftStar, ftPlanet);

  TUIButton = record
    x, y, texture: integer;
    focus: smallint;
    visible, enabled: boolean;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormClose(Sender: TObject; var {%H-}CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; {%H-}Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; {%H-}Shift: TShiftState);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; {%H-}Shift: TShiftState; {%H-}X, {%H-}Y: Integer);
    procedure FormMouseMove(Sender: TObject; {%H-}Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; {%H-}Button: TMouseButton; {%H-}Shift: TShiftState; {%H-}X, {%H-}Y: Integer);
    procedure FormMouseWheel(Sender: TObject; {%H-}Shift: TShiftState; WheelDelta: Integer; {%H-}MousePos: TPoint; var {%H-}Handled: Boolean);
    procedure Timer1Timer(Sender: TObject);
  private
    fb: TFrameBuffer;
    //grid: TVertexArray;
    bassSound: TBassEngine;
    shipDl: array[0..1] of TDisplayList;
    backDl, planetDl, warpDL, arrowDl: TDisplayList;
    mb, mx, my, buttonSize, focus: integer;
    button: array[0..4] of TUIButton;
    mapMode: boolean;
    focType: TFocusType;
    warpAlpha, plAnim, logoProgress: single;
  public
    procedure DrawScene;
    procedure DrawLogo;
    procedure DrawUI;
    procedure InitUI;
    procedure LoadGraphics;
  end;

var Form1: TForm1;

implementation

uses math, nxMath;

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  Randomize;
  ClientHeight:=screen.Height*4 div 5;
  ClientWidth:=ClientHeight*4 div 3;
  nxFARZ:=SectorSize*0.5;
  if not nx.CreateGlWindow(self) then begin
    showmessage(nx.LastError); exit;
  end;

  textLog:=TStringList.Create;
  game:=TGame.Create;

  bassSound:=TBassEngine.Create(handle);
  if bassSound.CheckErrors then begin
    showmessage(nx.LastError); exit;
  end;

  nx.DefaultLights;
  nx.SetLight(0, GL_AMBIENT, 0, 0, 0);
  nx.rs.DepthTest:=true;

  //nx.CreateFont('Blaster Laser', 11, 256);
  nx.CreateBasicFont;
  game.pt:=TParticleEngine.Create;
  LoadGraphics;
  InitUI;

  // Load and play music
  with bassSound.Add('music', 'sounds\k_worm.s3m') do begin
    looped:=true; Play;
  end;

  if nx.LastError<>'' then showmessage(nx.LastError)
  else timer1.Enabled:=true;
end;

procedure TForm1.LoadGraphics;
var i: integer; model: TGLModel; // j: integer;
begin
  //fb:=TFrameBuffer.Create(clientWidth, clientHeight, false);

  // Initialize grid
  {grid:=TVertexArray.Create(GS*GS*4, (GS+1)*(GS+1), GL_QUADS, true, false, false);
  for j:=0 to GS do
    for i:=0 to GS do begin
      // Texture coordinates
      grid.ta[2*(i+j*(GS+1))]:=i/GS;
      grid.ta[2*(i+j*(GS+1))+1]:=j/GS;

      // Vertex Z coordinate
      grid.va[i+j*(GS+1)].z:=0;
    end;

  // Face vertex indexes
  for j:=0 to GS-1 do
    for i:=0 to GS-1 do
      if i mod 2=0 then begin
        grid.fa[4*(i+j*GS)]:=i+j*(GS+1);
        grid.fa[4*(i+j*GS)+1]:=i+(j+1)*(GS+1);
        grid.fa[4*(i+j*GS)+2]:=i+1+(j+1)*(GS+1);
        grid.fa[4*(i+j*GS)+3]:=i+1+j*(GS+1);
      end else begin
        // Improve grid drawing by flipping every second quad
        grid.fa[4*(i+j*GS)]:=i+(j+1)*(GS+1);
        grid.fa[4*(i+j*GS)+1]:=i+1+(j+1)*(GS+1);
        grid.fa[4*(i+j*GS)+2]:=i+1+j*(GS+1);
        grid.fa[4*(i+j*GS)+3]:=i+j*(GS+1);
      end; }
  tex.AddTexture('glow', 'textures\glow0.png');
  tex.AddTexture('star', 'textures\star.png');
  tex.AddTexture('sun', 'textures\sun.png', true);
  tex.AddTexture('button', 'textures\button.png');
  for i:=0 to 4 do
    tex.AddTexture('button'+inttostr(i), 'textures\button'+inttostr(i)+'.png', true);
  for i:=0 to 2 do
    tex.AddTexture('back'+inttostr(i), 'textures\back'+inttostr(i)+'.png');
  for i:=0 to 9 do
    game.pType[i].texture:=tex.AddTexture('planet'+inttostr(i), 'textures\pt'+inttostr(i)+'.jpg');
  tex.AddTexture('focus', 'textures\focus.png', true);
  tex.AddTexture('focus2', 'textures\focus2.png', true);
  tex.AddTexture('warp', 'textures\warp.png');
  tex.AddTexture('warptarget', 'textures\warptarget.png');

  game.pt.AddPType(tex.IndexOf('glow'), 0.5, -0.03, -0.001);

  model:=TGLModel.Create;
  // Ships
  for i:=0 to high(shipDl) do begin
    model.LoadFromFile('objects\ship'+inttostr(i)+'.w3d');
    model.UseColors:=false;
    model.LoadTextures('textures');
    model.MakeDisplayList(shipDl[i]);
  end;
  // Planet
  model.LoadFromFile('objects\planet.w3d');
  model.UseMaterials:=false;
  model.MakeDisplayList(planetDl);
  // Background
  model.LoadFromFile('objects\back.w3d');
  model.UseMaterials:=false;
  model.MakeDisplayList(backDl);
  // Warp
  model.LoadFromFile('objects\warp.w3d');
  model.UseMaterials:=false;
  model.Scale(60,60,60);
  model.MakeDisplayList(warpDl);
  // Planet arrow
  model.LoadFromFile('objects\arrow.w3d');
  model.UseMaterials:=false;
  model.Scale(100,100,100);
  model.MakeDisplayList(arrowDl);
  model.Free;

  tex.AddTexture('logo', 'textures\logo.png');
end;

procedure TForm1.InitUI;
var i: integer;
begin
  focus:=-1;
  buttonSize:=ClientHeight div 10;
  for i:=0 to 4 do begin
    button[i].texture:=tex.IndexOf('button'+inttostr(i));
    button[i].enabled:=true;
    button[i].visible:=true;
    button[i].focus:=0;
    button[i].x:=round(ClientWidth div 2+(i-2.5)*(buttonSize*1.1));
    button[i].y:=ClientHeight-buttonSize*3 div 2;
  end;
  button[0].enabled:=false;
  button[1].visible:=false; // No use for comm button yet...
  button[2].enabled:=false;
  button[3].visible:=false; // No use for shipyard button yet...
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var i: integer; nearP: pPlanet;
begin
  if not nx.AllOK then begin
    timer1.Enabled:=false; showmessage(nx.LastError);
    exit;
  end;
  if game=nil then exit;

  if logoProgress>1.1 then game.GameLoop;
  nearP:=game.NearPlanet(plShip);

  for i:=0 to high(button) do
    if (button[i].focus>0) and (focus<>i) then begin
      button[i].focus:=button[i].focus-10;
      if button[i].focus<0 then button[i].focus:=0;
    end;

  DrawScene;

  nx.Enable2D;
  DrawUI;

  nx.SetColor(1, 1, 1);
  with nx.Font[0] do begin
    nx.SetFont(0);
    {Draw(10,10, format('FPS: %d', [nx.FPS]));
    with game.cam.position do
      Draw(10,25, format('Camera [%.2f, %.2f]', [x, y]));
    with game.cam.secLoc do
      Draw(10,40, format('Sector [%d, %d]', [x, y]));
    Draw(10,55, format('Planets here: %d', [game.CurSector^.planets]));
    Draw(10,70, format('Ships here: %d', [game.CurSector^.ships]));
    with curPos do
      Draw(10,85, format('Cursor [%.2f, %.2f]', [x, y]));}

    if plShip<>nil then begin
      // Sector name
      DrawScaled(ClientWidth-200, 20, 2, 2, format('%s',
        [game.sector[plShip.secLoc.x, plShip.secLoc.y].name]));
      // Planet info
      if nearP<>nil then begin
        Draw(ClientWidth-200, 60, 'Planet information:');
        Draw(ClientWidth-180, 80, format('%s',
          [pTypeStr[nearP^.pType]]));
        Draw(ClientWidth-180, 95, format('Size: %.2f',
          [nearP^.size]));
        Draw(ClientWidth-180, 110, format('Population: %.1f/%.1f',
          [nearP^.pop, game.MaxPop(nearP)]));
        case nearP^.owner of
          0: Draw(ClientWidth-180, 125, 'Owner: You');
          1..7: Draw(ClientWidth-180, 125, format('Owner: Hostile #%d',
               [nearP^.owner]));
          else Draw(ClientWidth-180, 125, 'Owner: <unclaimed>');
        end;
      end;
      // Targeting status
      if plShip.target<>nil then begin
        nx.SetColor(1, 0, 0, 0.7+0.3*sin(nx.GetTick2(360, 0.1)*toRad));
        DrawCScaled(ClientWidth/2, 20, 2, 2, '<TARGET LOCKED>');
      end;

      // Text log
      for i:=0 to TextLog.Count-1 do begin
        nx.SetColor(1,1,1, (i+1)/(TextLog.Count));
        DrawScaled(40, 200+i*20, 1.5, 1.5, format('%s',
          [textLog[i]]));
      end;
    end;
  end;

  nx.Disable2D;

  if logoProgress<2 then begin
    logoProgress:=logoProgress+0.005;
    DrawLogo;
  end;

  nx.Flip;
end;

procedure TForm1.DrawScene;
var i, j, k: integer; d, nearest: single; curSec: pSector;
    ppt: pParticle; nearP: pPlanet;
begin
  if warpAlpha<1 then warpAlpha:=warpAlpha+0.01;
  with game do begin
    curSec:=CurSector;
    nearP:=NearPlanet(plShip);

    // Set camera
    nx.Clear(true, true);
    glLoadIdentity;
    glTranslatef(0, 0, -cam.zoom);
    glrotatef(cam.rotY, 1,0,0);
    glrotatef(cam.rotX, 0,1,0);

    // Background
    glPushMatrix;
    nx.rs.Push;
    nx.rs.DepthTest:=false; nx.rs.Lighting:=false; nx.rs.AddBlend:=true;
    glScalef(1000, 1000, 1000);
    nx.SetColor(1, 1, 1);
    tex.SetByName('back'+inttostr(cursec^.back));
    for i:=0 to 3 do begin
      backDl.Draw;
      glRotatef(90, 0,1,0);
    end;
    tex.SetByName('back0');
    glRotatef(90, 1,0,0);
    backDl.Draw; // Top
    glRotatef(180, 1,0,0);
    backDl.Draw; // Bottom
    nx.rs.Pop;
    glPopMatrix;

    // Draw warp
    if (plShip<>nil) and (plShip.warping>0) then begin
      d:=3-abs(0.5-smoothen(plShip.warpProgress))*6;
      if d>0.8 then d:=0.8
      else if d<0 then d:=0;
      glPushMatrix; nx.rs.push;
      nx.rs.Lighting:=false;
      glMatrixMode(GL_TEXTURE);
      glPushMatrix;
      glTranslatef(0, -nx.GetTick*0.001, 0);
      glScalef(1/(1+d*2), 1/(1+d*4), 1);
      glMatrixMode(GL_MODELVIEW);

      nx.SetColor(0.5, 0.7, 1, d);
      tex.SetByName('warp');
      glrotatef(270-plShip.rotX, 0,1,0);
      warpDl.Draw;
      glMatrixMode(GL_TEXTURE);
      glPopMatrix;
      glMatrixMode(GL_MODELVIEW);
      nx.rs.pop; glPopMatrix;
    end;

    with cam.position do glTranslatef(-x, 0, -y);
    curPos:=nx.MouseRayAtPlane(mx, ClientHeight-1-my, vector(0,0,0), vector(0,1,0));

    nx.SetLight(0, GL_POSITION, SectorSize*0.5, 500, SectorSize*0.5);

    // Sun
    if curSec^.sun<>nil then begin
      nx.rs.Push;
      nx.rs.Lighting:=false; nx.rs.DepthTest:=false;
      nx.SetColor(curSec^.sun^.color);
      tex.SetByName('sun');
      glPushMatrix;
      glTranslatef(SectorSize*0.5, 500, SectorSize*0.5);
      glRotatef(90-angle(cam.position.x, cam.position.y,
        SectorSize*0.5, SectorSize*0.5)*toDeg, 0,1,0);
      glRotatef(0-angle(0, 0,
        hypot(SectorSize*0.5-cam.position.x, SectorSize*0.5-cam.position.y),
        500)*toDeg, 1,0,0);
      glScalef(5, 5, 5);
      nx.RectT(-60,-60,60,60);
      nx.rs.AddBlend:=true;
      tex.SetByName('glow');
      nx.SetColor(0.5,0.5,0.5);
      nx.RectT(-120,-40,120,40);
      glPopMatrix;
      nx.rs.Pop;

      // Planets
      nx.rs.Lighting:=true;
      nx.SetLight(1, GL_AMBIENT, 0, 0, 0);
      nx.SetColor(curSec^.sun^.color);
      for i:=0 to curSec^.planets-1 do
        with curSec^.planet[i] do begin
          glPushMatrix;
          with position do glTranslatef(x, -size*1.1, y);
          glScalef(size, size, size);
          tex.SetTex(game.pType[pType].texture);
          planetDl.Draw;
          glPopMatrix;
        end;
      nx.SetLight(1, GL_AMBIENT, 0.4, 0.4, 0.4);

      if nearP<>nil then
        with nearP^ do begin
          nx.SetColor(0.3, 1, 1, 0.5+0.5*sin(nx.GetTick2(360, 0.1)*toRad));
          tex.Disable; glPushMatrix;
          with position do glTranslatef(x, 50, y);
          arrowDl.Draw;
          tex.Enable; glPopMatrix;
        end;
    end;

    // Parallax
    tex.Disable;
    glPushMatrix;
    with cam.position do glTranslatef(x, 0, y);
    glPointSize(2);
    glBegin(GL_POINTS);
    for i:=0 to ParallaxCount-1 do
      with parallax[i] do begin
        d:=1.5-max(abs(position.x),
                   abs(position.z)) / (ParallaxDistance/2);
        if d>1 then d:=1
        else if d<0 then d:=0;
        nx.SetColor(color.r/255, color.g/255, color.b/255, d);
        glVertex3fv(@position);
      end;
    glEnd;
    glPopMatrix;
    tex.Enable;

    // Ships
    if curSec^.sun<>nil then nx.SetColor(curSec^.sun^.color)
    else nx.SetColor(0.7, 0.7, 0.7);
    for j:=0 to high(pl) do
      for i:=0 to pl[j].ships-1 do
        with pl[j].ship[i] do
          if (cam.secLoc.x=secLoc.x) and (cam.secLoc.y=secLoc.y) then begin
            glPushMatrix;
            with position do glTranslatef(x, 0, y);
            glrotatef(rotZ, 0,0,1);
            glrotatef(270-rotX, 0,1,0);
            shipDl[sType].Draw;
            glPopMatrix;
            // Add ship engine particles
            if accelerating then
              for k:=-5 to 5 do
                if stype=0 then begin
                  ppt:=pt.AddParticle(0, 0.5, 1, 1, 0.25,
                    position.x-cos((rotX+k*8)*toRad)*0.8, 0,
                    position.y-sin((rotX+k*8)*toRad)*0.8);
                  ppt^.velocity:=vector(
                    movement.x-cos(rotX*toRad)*0.08, 0,
                    movement.y-sin(rotX*toRad)*0.08);
                end else if abs(k)<4 then begin
                  ppt:=pt.AddParticle(0, 0.5, 1, 1, 0.15,
                    position.x-cos((rotX+k*12)*toRad)*0.4, 0,
                    position.y-sin((rotX+k*12)*toRad)*0.4);
                  ppt^.velocity:=vector(
                    movement.x-cos(rotX*toRad)*0.08, 0,
                    movement.y-sin(rotX*toRad)*0.08);
                end;
          end;

    // Particles
    nx.rs.Push;
    nx.rs.Lighting:=false; nx.rs.AddBlend:=true;
    glDepthMask(false);
    pt.Draw3D;
    pt.Move3D;

    // Beams
    glLineWidth(2);
    tex.Disable;
    glBegin(GL_LINES);
    for i:=0 to beamCount-1 do
      with beam[i] do begin
        nx.SetColor(color.r/255, color.r/255, color.r/255, color.a/500);
        glVertex3f(p1.x, 0, p1.y);
        nx.SetColor(color);
        glVertex3f(p2.x, 0, p2.y);
      end;
    glEnd;
    tex.Enable;

    glDepthMask(true);
    nx.rs.Pop;

    // Map
    if mapMode then begin
      case focType of
        ftStar, ftPlanet: focType:=ftNone;
      end;

      nx.Enable2D;
      // Darken background
      tex.Disable; nx.SetColor(0,0,0, 0.85);
      nx.RectT(0, 0, ClientWidth, ClientHeight);
      tex.Enable;

      // Set view
      nx.rs.AddBlend:=true;
      if not systemView then begin
        // Star map
        glTranslatef(clientwidth/2, 0, 0);
        glScalef(clientheight*0.8/(WorldSize+2), clientheight*0.8/(WorldSize+2), 1);
        glTranslatef(-WorldSize/2, 2, 0);
        curPos:=nx.MouseRayAtPlane(mx, ClientHeight-1-my, vector(0,0,0),
          vector(0,0,1));
        focX:=-1; focY:=-1;

        // Focus star with cursor
        if focType=ftNone then begin
          nearest:=2;
          for j:=0 to WorldSize-1 do
            for i:=0 to WorldSize-1 do
              with sector[i, j] do
                if sun<>nil then begin
                  d:=hypot(i-curPos.x, j-curPos.y);
                  if d<nearest then begin
                    nearest:=d; focX:=i; focY:=j;
                    focType:=ftStar;
                  end;
                end;
        end;

        tex.SetByName('focus2');
        // Draw focus on system pointed by cursor
        if focType=ftStar then begin
          nx.SetColor(0, 1, 1, 0.3);
          d:=nx.GetTick2(360, 0.05);
          nx.DrawRotateS(focX, focY, 0, d, 0.5, 0.5, 2, 2);
          nx.DrawRotateS(focX, focY, 0, -d*2, 0.5, 0.5, 2, 2);
        end;
        // Draw map opening anim on current system
        if plAnim<1 then begin
          nx.SetColor(1, 1, 1, sin((plAnim*180)*toRad));
          d:=nx.GetTick2(360, 0.1);
          nx.DrawRotateS(cam.secLoc.x, cam.secLoc.y, 0, d, 0.5, 0.5,
            (1-plAnim)*20, (1-plAnim)*20);
          plAnim:=plAnim+0.01;
          if (plShip<>nil) and (plShip.warping>0) and (plAnim>=1) then
            plAnim:=0;
        end;

        // Stars in main map
        tex.SetByName('star');
        for j:=0 to WorldSize-1 do
          for i:=0 to WorldSize-1 do
            with sector[i, j] do
              if sun<>nil then begin
                nx.SetColor(sun^.color);
                nx.RectT(i-1.6, j-1.6, i+1.6, j+1.6);
              end;

        // Sector names
        nx.Font[0].SetTexture;
        for j:=0 to WorldSize-1 do
          for i:=0 to WorldSize-1 do
            with sector[i, j] do
              if (sun<>nil) and known then begin
                if (i=focX) and (j=focY) then nx.SetColor(1,1,1)
                //else if (i=cam.secLoc.x) and (j=cam.secLoc.y) then
                //  nx.SetColor(0,1,1)
                //else if (pl[0].warpSec.x=i) and (pl[0].warpSec.y=j) then
                //  nx.SetColor(0.5,0.2,1)
                else if game.IsEnemyPlanets(GetSecLoc(i,j), 0) then
                  nx.SetColor(1,0,0, 1)
                else if game.IsPlayerSector(GetSecLoc(i,j)) then
                  nx.SetColor(0,1,0, 1)
                else nx.SetColor(1,1,1, 0.4);
                nx.font[0].DrawCScaled(i, j+1.5, 0.08, 0.08, name);
              end;

      end else begin
        // System map
        glScalef(clientWidth/SectorSize, clientHeight*0.8/SectorSize, 1);
        curPos:=nx.MouseRayAtPlane(mx, ClientHeight-1-my, vector(0,0,0),
          vector(0,0,1));
        if curPos.x<0 then curPos.x:=0
        else if curPos.x>SectorSize then curPos.x:=SectorSize;
        if curPos.y<0 then curPos.y:=0
        else if curPos.y>SectorSize then curPos.y:=SectorSize;
        if pl[0].warpSec.x>-1 then
          if (pl[0].warpSec.x=focX) and (pl[0].warpSec.y=focY) then begin
            tex.SetByName('warptarget');
            with pl[0].warpPos do begin
              nx.SetColor(0.4, 0.2, 1, warpAlpha);
              nx.DrawRotateS(x, y, 0, nx.GetTick2(360, 0.01),
                0.5, 0.5, 13000, 12000);
              nx.SetColor(0.2, 0.3, 1, warpAlpha);
              nx.DrawRotateS(x, y, 0, -nx.GetTick2(360, 0.027),
                0.5, 0.5, 6000, 8000);
            end;
          end;

        with sector[focX, focY] do
          if sun<>nil then begin
            tex.SetByName('star');
            nx.SetColor(sun^.color);
            nx.DrawRotateS(SectorSize/2, SectorSize/2, 0, 0, 0.5, 0.5,
              13000, 13000);

            // Show ships in sector
            if (cam.secLoc.X=focX) and (cam.secLoc.Y=focY) then begin
              tex.Disable;
              glBegin(GL_LINES);
              for j:=0 to high(pl) do
                for i:=0 to pl[j].ships-1 do
                  with pl[j].ship[i] do
                    if (secloc.x=focX) and (secloc.y=focY) then begin
                      if j=0 then
                        nx.SetColor(0, 1, 0, 0.4+0.3*sin(nx.GetTick2(360, 0.027)*toRad))
                      else
                        nx.SetColor(1, 0, 0, 0.4+0.3*sin(nx.GetTick2(360, 0.027)*toRad));
                      glVertex2f(0, position.y);
                      glVertex2f(SectorSize, position.y);
                      glVertex2f(position.x, 0);
                      glVertex2f(position.x, SectorSize);
                    end;
              glEnd;
              tex.Enable;
            end;

            // Planets
            tex.SetByName('focus');
            for i:=0 to planets-1 do begin
              if known then begin
                if planet[i].owner=0 then nx.SetColor(0.3, 1, 0.3)
                else if planet[i].owner>0 then nx.SetColor(1, 0.3, 0.3)
                else nx.SetColor(0.7, 0.7, 0.7);
              end else nx.SetColor(0.3, 0.3, 1);
              nx.DrawRotateS(planet[i].position.x, planet[i].position.y,
                0, 0, 0.5, 0.5, 2000, 2000);
            end;
          end;
        // Cursor
        tex.SetByName('focus2');
        nx.SetColor(1, 1, 1, sin(nx.GetTick2(360, 0.1)*toRad)*0.3+0.7);
        nx.DrawRotateS(curPos.x, curPos.y, 0, nx.GetTick2(360, 0.1),
          0.5, 0.5, 5000, 5000);
      end;
      nx.rs.AddBlend:=false;

      nx.Disable2D;
    end;

  end; // with game
end;

procedure TForm1.DrawLogo;
begin
  nx.Enable2D;
  // Faded black background
  tex.Disable;
  nx.SetColor(0,0,0, 2-logoProgress);
  nx.RectT(0,0, ClientWidth, ClientHeight);
  tex.Enable;

  // Logo
  nx.rs.AddBlend:=true;
  tex.SetByName('logo');
  nx.SetColor(1,1,1, (2-logoProgress)*logoProgress);
  nx.DrawRotateS(ClientWidth/2, ClientHeight/2, 0, 0, 0.5, 0.5,
    ClientWidth*0.8, ClientWidth*(0.8/8));
  nx.SetColor(0,0.3,1, (1.5-logoProgress)*logoProgress);
  nx.DrawRotateS(ClientWidth/2, ClientHeight/2, 0, 0, 0.5, 0.5,
    ClientWidth*(0.8+logoProgress*0.5), ClientWidth*(0.85/8)/(1+logoProgress));
  nx.rs.AddBlend:=false;
  nx.Disable2D;
end;

procedure TForm1.DrawUI;
var i: integer; d, d2: single;
begin
  button[2].enabled:=(plShip<>nil) and (game.pl[0].warpSec.x>-1) and (plShip.warping=0);
  button[0].enabled:=(plShip<>nil) and (not game.pl[0].inCombat);

  glPushMatrix;
  d2:=nx.GetTick2(360, 0.3)*toRad;
  for i:=0 to high(button) do
    with button[i] do
      if visible then begin
        if enabled then begin
          d:=focus/255;
          nx.SetColor(0.8-0.6*d, 0.8+0.2*d, 0.8+0.2*d); // Teal
        end else
          nx.SetColor(0.5, 0.5, 0.5, 0.3);
        tex.SetTex(texture);
        nx.DrawScaled(x, y, buttonSize, buttonSize);
      end;
  nx.rs.AddBlend:=true;
  tex.SetByName('button');
  for i:=0 to high(button) do
    with button[i] do
      if visible and enabled then begin
        if focus>200 then nx.SetColor(1, 1, 1, 0.8+0.2*sin(d2))
        else nx.SetColor(1, 1, 1);
        nx.DrawScaled(x-2, y-2, buttonSize+4, buttonSize+4);
      end;
  nx.rs.AddBlend:=false;
  glPopMatrix;
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var i: integer;
begin
  if button=mbLeft then begin
    if focType=ftButton then begin
      case TUIButtonType(focus) of
        btAttack: begin
            game.pl[0].inCombat:=true;
          end;
        btWarp: if game.pl[0].warpSec.x>-1 then begin
            with game.pl[0] do begin
              inCombat:=false;
              for i:=0 to ships-1 do
                if ship[i].inFleet then
                  game.InitiateWarp(ship[i], game.pl[0].warpSec,
                    game.pl[0].warpPos);
              with game.sector[game.pl[0].warpSec.x, game.pl[0].warpSec.y] do begin
                if known and (name<>'') then
                  game.AddLog('Sending fleet to '+name)
                else game.AddLog('Sending fleet to unknown system');
              end;
            end;
          end;
        btMap: begin
            mapMode:=not mapMode;
            if MapMode and (not systemView) then plAnim:=0;
          end;
      end;
    end else if mapMode and (focType=ftStar) and (not systemView) then begin
      systemView:=true;
    end else if mapMode and systemView then begin
      game.pl[0].warpSec:=getSecLoc(focX, focY);
      game.pl[0].warpPos.x:=curPos.x;
      game.pl[0].warpPos.y:=curPos.y;
      warpAlpha:=0;
    end else
      mb:=1;
  end else begin
    if mapMode then begin
      systemView:=not systemView;
      if SystemView then begin
        focX:=game.cam.secLoc.x; focY:=game.cam.secLoc.y;
      end else begin
        plAnim:=0;
      end;
    end else mb:=2;
  end;
end;

procedure TForm1.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var dx, dy: single; i: integer;
begin
  dx:=X-mx; dy:=Y-my; mx:=X; my:=Y;
  if mb>0 then begin
    with game.cam do begin
      if not mapmode then begin
        toRotX:=toRotX+dx*0.6;
        toRotY:=toRotY+dy*0.8;
        if toRotX<0 then toRotX:=toRotX+360
        else if toRotX>=360 then toRotX:=toRotX-360;
        if toRotY>90 then toRotY:=90
        else if toRotY<-90 then toRotY:=-90;
      end;
    end;
  end else begin
    focus:=-1; focType:=ftNone;
    for i:=0 to high(button) do
      with button[i] do
        if visible and enabled then
          if PointInCircle(mx, my, x+buttonSize/2, y+buttonSize/2, buttonSize/2) then begin
            self.focus:=i; focus:=255; focType:=ftButton;
            break;
          end;
  end;
end;

procedure TForm1.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  mb:=0;
end;

procedure TForm1.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  with game.cam do begin
    toZoom:=toZoom-(WheelDelta div abs(wheelDelta));
    if toZoom<2 then toZoom:=2
    else if toZoom>25 then toZoom:=25;
  end;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  keys[key]:=true;
  if key=ord('M') then begin
    MapMode:=not MapMode;
    if mapMode and (not systemView) then plAnim:=0;
  end;
  //caption:=inttostr(key)+' '+inttostr(ord('W'));
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  keys[key]:=false;
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
var i: integer;
begin
  timer1.Enabled:=false;
  if fb<>nil then fb.Free;
  if game<>nil then game.pt.Free;
  for i:=0 to high(shipDl) do shipDl[i].Free;
  backDl.Free; planetDl.Free; warpDl.Free; arrowDl.Free;
  if bassSound.count>0 then begin
    bassSound.sound[0].FadeVolume(0, 50);
    sleep(50);
  end;
  FreeAndNil(bassSound);
  nx.KillGLWindow;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if game<>nil then game.Free;
  textLog.Free;
end;

end.

